@@ -55,90 +55,6 @@ gcry_mpi_t 				gk, myk, sig_r, sig_s;
 
 int 					opt_verbose = 0;
 
-#define TRIVIAL_SIZE 1024
-#define PRIMES_SIZE 669
-unsigned long int 			primes[] = {
-						2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43,
-						47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97, 101,
-						103, 107, 109, 113, 127, 131, 137, 139, 149, 151,
-						157, 163, 167, 173, 179, 181, 191, 193, 197, 199,
-						211, 223, 227, 229, 233, 239, 241, 251, 257, 263,
-						269, 271, 277, 281, 283, 293, 307, 311, 313, 317,
-						331, 337, 347, 349, 353, 359, 367, 373, 379, 383,
-						389, 397, 401, 409, 419, 421, 431, 433, 439, 443,
-						449, 457, 461, 463, 467, 479, 487, 491, 499, 503,
-						509, 521, 523, 541, 547, 557, 563, 569, 571, 577,
-						587, 593, 599, 601, 607, 613, 617, 619, 631, 641,
-						643, 647, 653, 659, 661, 673, 677, 683, 691, 701,
-						709, 719, 727, 733, 739, 743, 751, 757, 761, 769,
-						773, 787, 797, 809, 811, 821, 823, 827, 829, 839,
-						853, 857, 859, 863, 877, 881, 883, 887, 907, 911,
-						919, 929, 937, 941, 947, 953, 967, 971, 977, 983,
-						991, 997, 1009, 1013, 1019, 1021, 1031, 1033,
-						1039, 1049, 1051, 1061, 1063, 1069, 1087, 1091,
-						1093, 1097, 1103, 1109, 1117, 1123, 1129, 1151,
-						1153, 1163, 1171, 1181, 1187, 1193, 1201, 1213,
-						1217, 1223, 1229, 1231, 1237, 1249, 1259, 1277,
-						1279, 1283, 1289, 1291, 1297, 1301, 1303, 1307,
-						1319, 1321, 1327, 1361, 1367, 1373, 1381, 1399,
-						1409, 1423, 1427, 1429, 1433, 1439, 1447, 1451,
-						1453, 1459, 1471, 1481, 1483, 1487, 1489, 1493,
-						1499, 1511, 1523, 1531, 1543, 1549, 1553, 1559,
-						1567, 1571, 1579, 1583, 1597, 1601, 1607, 1609,
-						1613, 1619, 1621, 1627, 1637, 1657, 1663, 1667,
-						1669, 1693, 1697, 1699, 1709, 1721, 1723, 1733,
-						1741, 1747, 1753, 1759, 1777, 1783, 1787, 1789,
-						1801, 1811, 1823, 1831, 1847, 1861, 1867, 1871,
-						1873, 1877, 1879, 1889, 1901, 1907, 1913, 1931,
-						1933, 1949, 1951, 1973, 1979, 1987, 1993, 1997,
-						1999, 2003, 2011, 2017, 2027, 2029, 2039, 2053,
-						2063, 2069, 2081, 2083, 2087, 2089, 2099, 2111,
-						2113, 2129, 2131, 2137, 2141, 2143, 2153, 2161,
-						2179, 2203, 2207, 2213, 2221, 2237, 2239, 2243,
-						2251, 2267, 2269, 2273, 2281, 2287, 2293, 2297,
-						2309, 2311, 2333, 2339, 2341, 2347, 2351, 2357,
-						2371, 2377, 2381, 2383, 2389, 2393, 2399, 2411,
-						2417, 2423, 2437, 2441, 2447, 2459, 2467, 2473,
-						2477, 2503, 2521, 2531, 2539, 2543, 2549, 2551,
-						2557, 2579, 2591, 2593, 2609, 2617, 2621, 2633,
-						2647, 2657, 2659, 2663, 2671, 2677, 2683, 2687,
-						2689, 2693, 2699, 2707, 2711, 2713, 2719, 2729,
-						2731, 2741, 2749, 2753, 2767, 2777, 2789, 2791,
-						2797, 2801, 2803, 2819, 2833, 2837, 2843, 2851,
-						2857, 2861, 2879, 2887, 2897, 2903, 2909, 2917,
-						2927, 2939, 2953, 2957, 2963, 2969, 2971, 2999,
-						3001, 3011, 3019, 3023, 3037, 3041, 3049, 3061,
-						3067, 3079, 3083, 3089, 3109, 3119, 3121, 3137,
-						3163, 3167, 3169, 3181, 3187, 3191, 3203, 3209,
-						3217, 3221, 3229, 3251, 3253, 3257, 3259, 3271,
-						3299, 3301, 3307, 3313, 3319, 3323, 3329, 3331,
-						3343, 3347, 3359, 3361, 3371, 3373, 3389, 3391,
-						3407, 3413, 3433, 3449, 3457, 3461, 3463, 3467,
-						3469, 3491, 3499, 3511, 3517, 3527, 3529, 3533,
-						3539, 3541, 3547, 3557, 3559, 3571, 3581, 3583,
-						3593, 3607, 3613, 3617, 3623, 3631, 3637, 3643,
-						3659, 3671, 3673, 3677, 3691, 3697, 3701, 3709,
-						3719, 3727, 3733, 3739, 3761, 3767, 3769, 3779,
-						3793, 3797, 3803, 3821, 3823, 3833, 3847, 3851,
-						3853, 3863, 3877, 3881, 3889, 3907, 3911, 3917,
-						3919, 3923, 3929, 3931, 3943, 3947, 3967, 3989,
-						4001, 4003, 4007, 4013, 4019, 4021, 4027, 4049,
-						4051, 4057, 4073, 4079, 4091, 4093, 4099, 4111,
-						4127, 4129, 4133, 4139, 4153, 4157, 4159, 4177,
-						4201, 4211, 4217, 4219, 4229, 4231, 4241, 4243,
-						4253, 4259, 4261, 4271, 4273, 4283, 4289, 4297,
-						4327, 4337, 4339, 4349, 4357, 4363, 4373, 4391,
-						4397, 4409, 4421, 4423, 4441, 4447, 4451, 4457,
-						4463, 4481, 4483, 4493, 4507, 4513, 4517, 4519,
-						4523, 4547, 4549, 4561, 4567, 4583, 4591, 4597,
-						4603, 4621, 4637, 4639, 4643, 4649, 4651, 4657,
-						4663, 4673, 4679, 4691, 4703, 4721, 4723, 4729,
-						4733, 4751, 4759, 4783, 4787, 4789, 4793, 4799,
-						4801, 4813, 4817, 4831, 4861, 4871, 4877, 4889,
-						4903, 4909, 4919, 4931, 4933, 4937, 4943, 4951,
-						4957, 4967, 4969, 4973, 4987, 4993, 4999, 0
-};
-
 int main
 	(int argc, char *const *argv)
 {
@@ -394,71 +310,7 @@ int main
 	std::cout << "|g| = " << mpz_sizeinbase(dss->g, 2L) << " bit";
 	if (dss_n != 0)
 		std::cout << ", |h| = " << mpz_sizeinbase(dss->h, 2L) << " bit";
-	std::cout << std::endl << "\t";
-	std::cout << "p is ";
-	if (!mpz_probab_prime_p(dss->p, TMCG_MR_ITERATIONS))
-		std::cout << "NOT ";
-	std::cout << "probable prime" << std::endl << "\t";
-	mpz_t pm1;
-	mpz_init_set(pm1, dss->p);
-	mpz_sub_ui(pm1, pm1, 1L);
-	std::cout << "(p-1) = ";
-	for (size_t i = 0; i < PRIMES_SIZE; i++)
-	{
-		if (mpz_divisible_ui_p(pm1, primes[i]))
-			std::cout << primes[i] << " * ";
-	}
-	std::cout << "...";
-	if (mpz_divisible_p(pm1, dss->q))
-		std::cout << " * q";
-	std::cout << std::endl << "\t";
-	std::cout << "q is ";
-	if (!mpz_probab_prime_p(dss->q, TMCG_MR_ITERATIONS))
-		std::cout << "NOT ";
-	std::cout << "probable prime" << std::endl << "\t";
-	mpz_set(pm1, dss->q);
-	mpz_sub_ui(pm1, pm1, 1L);
-	std::cout << "(q-1) = ";
-	for (size_t i = 0; i < PRIMES_SIZE; i++)
-	{
-		if (mpz_divisible_ui_p(pm1, primes[i]))
-			std::cout << primes[i] << " * ";
-	}
-	std::cout << "..." << std::endl << "\t";
-	std::cout << "g is ";
-	mpz_powm(pm1, dss->g, dss->q, dss->p);
-	if (mpz_cmp_ui(pm1, 1L))
-		std::cout << "NOT ";
-	std::cout << "generator of G_q" << std::endl << "\t";
-	std::cout << "h is ";
-	mpz_powm(pm1, dss->h, dss->q, dss->p);
-	if (mpz_cmp_ui(pm1, 1L))
-		std::cout << "NOT ";
-	std::cout << "generator of G_q" << std::endl;
-	std::cout << "Security level of public key: " << std::endl << "\t";
-	std::cout << "|y| = " << mpz_sizeinbase(dss->y, 2L) << " bit";
-	std::cout << std::endl << "\t";
-	std::cout << "y is ";
-	mpz_powm(pm1, dss->y, dss->q, dss->p);
-	if (mpz_cmp_ui(pm1, 1L))
-		std::cout << "NOT ";
-	std::cout << "element of G_q" << std::endl << "\t";
-	bool trivial = false;
-	for (size_t i = 0; i < TRIVIAL_SIZE; i++)
-	{
-		mpz_powm_ui(pm1, dss->g, i, dss->p);
-		if (!mpz_cmp(dss->y, pm1))
-		{
-			trivial = true;
-			break;
-		}
-	}
-	if (!trivial)
-		std::cout << "y is not trivial" << std::endl << "\t";
-	else
-		std::cout << "y IS TRIVIAL, i.e., y = g^c mod p (for some c < " << TRIVIAL_SIZE << ")" << std::endl << "\t";
-	std::cout << "Legendre-Jacobi symbol (y/p) is " << mpz_jacobi(dss->y, dss->p) << std::endl;
-	mpz_clear(pm1);
+	std::cout << std::endl;
 	if (dss_n != 0)
 	{
 		std::cout << "Threshold parameter set of primary key (tDSS): " << std::endl << "\t";
@@ -479,6 +331,12 @@ int main
 		std::cout << "Canonicalized peer list (CAPL): " << std::endl;
 		for (size_t i = 0; i < CAPL.size(); i++)
 			std::cout << "\t" << "P_" << ((dss->dkg->x_rvss->QUAL.size())?(i):(dss->QUAL[i])) << "\t" << CAPL[i] << std::endl;
+		std::cout << "Public commitments C_ik of x_rvss subprotocol: " << std::endl;
+		for (size_t i = 0; i < dss->dkg->x_rvss->C_ik.size(); i++)
+		{
+			for (size_t k = 0; k < dss->dkg->x_rvss->C_ik[i].size(); k++)
+				std::cout << "\t" << "C_ik[" << i << "][" << k << "] = " << dss->dkg->x_rvss->C_ik[i][k] << std::endl;
+		}
 	}
 	if (sub.size() && (dkg != NULL))
 	{
@@ -498,70 +356,7 @@ int main
 		std::cout << "|p| = " << mpz_sizeinbase(dkg->p, 2L) << " bit, ";
 		std::cout << "|q| = " << mpz_sizeinbase(dkg->q, 2L) << " bit, ";
 		std::cout << "|g| = " << mpz_sizeinbase(dkg->g, 2L) << " bit, ";
-		std::cout << "|h| = " << mpz_sizeinbase(dkg->h, 2L) << " bit" << std::endl << "\t";
-		std::cout << "p is ";
-		if (!mpz_probab_prime_p(dkg->p, TMCG_MR_ITERATIONS))
-			std::cout << "NOT ";
-		std::cout << "probable prime" << std::endl << "\t";
-		mpz_init_set(pm1, dkg->p);
-		mpz_sub_ui(pm1, pm1, 1L);
-		std::cout << "(p-1) = ";
-		for (size_t i = 0; i < PRIMES_SIZE; i++)
-		{
-			if (mpz_divisible_ui_p(pm1, primes[i]))
-				std::cout << primes[i] << " * ";
-		}
-		std::cout << "...";
-		if (mpz_divisible_p(pm1, dkg->q))
-			std::cout << " * q";
-		std::cout << std::endl << "\t";
-		std::cout << "q is ";
-		if (!mpz_probab_prime_p(dkg->q, TMCG_MR_ITERATIONS))
-			std::cout << "NOT ";
-		std::cout << "probable prime" << std::endl << "\t";
-		mpz_set(pm1, dkg->q);
-		mpz_sub_ui(pm1, pm1, 1L);
-		std::cout << "(q-1) = ";
-		for (size_t i = 0; i < PRIMES_SIZE; i++)
-		{
-			if (mpz_divisible_ui_p(pm1, primes[i]))
-				std::cout << primes[i] << " * ";
-		}
-		std::cout << "..." << std::endl << "\t";
-		std::cout << "g is ";
-		mpz_powm(pm1, dkg->g, dkg->q, dkg->p);
-		if (mpz_cmp_ui(pm1, 1L))
-			std::cout << "NOT ";
-		std::cout << "generator of G_q" << std::endl << "\t";
-		std::cout << "h is ";
-		mpz_powm(pm1, dkg->h, dkg->q, dkg->p);
-		if (mpz_cmp_ui(pm1, 1L))
-			std::cout << "NOT ";
-		std::cout << "generator of G_q" << std::endl;
-		std::cout << "Security level of public key: " << std::endl << "\t";
-		std::cout << "|y| = " << mpz_sizeinbase(dkg->y, 2L) << " bit";
-		std::cout << std::endl << "\t";
-		std::cout << "y is ";
-		mpz_powm(pm1, dkg->y, dkg->q, dkg->p);
-		if (mpz_cmp_ui(pm1, 1L))
-			std::cout << "NOT ";
-		std::cout << "element of G_q" << std::endl << "\t";
-		trivial = false;
-		for (size_t i = 0; i < TRIVIAL_SIZE; i++)
-		{
-			mpz_powm_ui(pm1, dkg->g, i, dkg->p);
-			if (!mpz_cmp(dkg->y, pm1))
-			{
-				trivial = true;
-				break;
-			}
-		}
-		if (!trivial)
-			std::cout << "y is not trivial" << std::endl << "\t";
-		else
-			std::cout << "y IS TRIVIAL, i.e., y = g^c mod p (for some c < " << TRIVIAL_SIZE << ")" << std::endl << "\t";
-		std::cout << "Legendre-Jacobi symbol (y/p) is " << mpz_jacobi(dkg->y, dkg->p) << std::endl;
-		mpz_clear(pm1);
+		std::cout << "|h| = " << mpz_sizeinbase(dkg->h, 2L) << " bit" << std::endl;
 		std::cout << "Threshold parameter set of subkey (DKG): " << std::endl << "\t";
 		std::cout << "n = " << dkg->n << ", t = " << dkg->t << std::endl;
 		std::cout << "Set of non-disqualified parties of subkey (DKG): " << std::endl << "\t" << "QUAL = { ";
@@ -573,6 +368,12 @@ int main
 		std::cout << "Public verification keys (DKG): " << std::endl;
 		for (size_t i = 0; i < dkg->v_i.size(); i++)
 			std::cout << "\t" << "v_" << i << " = " << dkg->v_i[i] << std::endl;
+		std::cout << "Public commitments C_ik (DKG): " << std::endl;
+		for (size_t i = 0; i < dkg->C_ik.size(); i++)
+		{
+			for (size_t k = 0; k < dkg->C_ik[i].size(); k++)
+				std::cout << "\t" << "C_ik[" << i << "][" << k << "] = " << dkg->C_ik[i][k] << std::endl;
+		}
 	}
 
 	// restore default formatting
